#version 330 core
struct Material
{
    sampler2D texture_diffuse1;
    sampler2D texture_specular1;
    float shininess;
}; 

struct DirLight
{
    vec3 direction;
    vec3 intensity;
};

#define NR_POINT_LIGHTS 1

in vec3 FragPosition;
in vec3 Normal;
in vec2 TexCoords;

out vec4 color;

uniform vec3 viewPos;
uniform DirLight dirLight;
uniform Material material;

// Function prototypes
vec3 CalcDirLight(DirLight light, Material mat, vec3 normal, vec3 fragPos, vec3 viewDir);

void main()
{    
    vec3 result;
    vec3 viewDir = normalize(viewPos - FragPosition);
    vec3 norm = normalize(Normal);
    
    result = CalcDirLight(dirLight, material, norm, FragPosition, viewDir);
        
    color = vec4(result, 1.0f);
}


// Calculates the color when using a dir light.
vec3 CalcDirLight(DirLight light, Material mat, vec3 normal, vec3 fragPos, vec3 viewDir)
{
    vec3 lightDir = normalize(-light.direction);
    // Diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // Specular shading
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), mat.shininess);
    // Attenuation
    //float distance = length(light.position - fragPos);
    //float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));    
    // Combine results
    vec3 ambient = light.intensity * vec3(texture(mat.texture_diffuse1, TexCoords));
    vec3 diffuse = light.intensity * diff * vec3(texture(mat.texture_diffuse1, TexCoords));
    vec3 specular = light.intensity * spec * vec3(texture(mat.texture_specular1, TexCoords));
    //ambient *= attenuation;
    //diffuse *= attenuation;
    //specular *= attenuation;
    return (ambient + diffuse + specular);
}