#version 330 core
in vec2 texCoord;
out vec4 color;

uniform sampler2D screenTexture;
uniform float width;
uniform float height;


void main()
{
/*
	// sobel-operator in x direction (partial derivative g_x)
	mat3 gX =  mat3( -1.0, 0.0, 1.0,
				   -2.0, 0.0, 2.0,
                   -1.0, 0.0, 1.0 );
	// sobel-operator in y direction (partial derivative g_x)
	mat3 GY =  mat3( 1.0,  2.0,  1.0,
                   0.0,  0.0,  0.0,
                   -1.0, -2.0, -1.0 );
*/
	float offsetX = 1.0f / width;
	float offsetY = 1.0f / height;
	
	vec3 threshold = vec3(0.8f, 0.8f, 0.8f);
	
	vec2 offsets[9] = vec2[](
		vec2(-offsetX, offsetY),  // top-left
        vec2(0.0f,    offsetY),  // top-center
        vec2(offsetX,  offsetY),  // top-right
        vec2(-offsetX, 0.0f),    // center-left
        vec2(0.0f,    0.0f),    // center-center
        vec2(offsetX,  0.0f),    // center-right
        vec2(-offsetX, -offsetY), // bottom-left
        vec2(0.0f,    -offsetY), // bottom-center
        vec2(offsetX,  -offsetY)  // bottom-right  
	);
	
	// sobel-operator in x direction (partial derivative g_x)
	float sobelX[9] = float[](
        -1, 0, 1,
        -2, 0, 2,
        -1, 0, 1
    );
	// sobel-operator in y direction (partial derivative g_x)
	float sobelY[9] = float[](
         1,  2, 1,
         0,  0, 0,
        -1, -2, -1
    );
	
	// get fragment neighbourhood
	vec3 texSample[9];
	for (int i = 0; i < 9; i++)
	{
		texSample[i] = vec3(texture(screenTexture, texCoord.st + offsets[i]));
	}
    
	// convolution in  x
	vec3 sumX = vec3(0.0);
	for (int i = 0; i < 9; i++)
	{
		sumX += texSample[i] * sobelX[i];
	}
	// convolution in y
	vec3 sumY = vec3(0.0);
	for (int i = 0; i < 9; i++)
	{
		sumY += texSample[i] * sobelY[i];
	}
	
	vec3 magnitude = sqrt( (sumX * sumX) + (sumY * sumY));
	
	float magLength = length(magnitude);
	
	if (magLength > 0.75)
		color = vec4(0.4, 0.4, 0.4, 1.0);
	else
		color = texture(screenTexture, texCoord); 
	
	//color = vec4(magnitude, 1.0f);
}