// shadertype = glsl
#version 330 core

layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texCoords;
layout (location = 3) in ivec4 boneIDs;
layout (location = 4) in vec4 weights;

out vec3 fragPos;
out vec3 Normal;
out float detail;

#define MAX_BONES 100

uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;
uniform mat3 normalMat;
uniform mat4 bones[MAX_BONES];

uniform vec3 viewPos;
uniform float magnitude;

void main()
{
	mat4 boneTransform = bones[boneIDs[0]] * weights[0];
	boneTransform += (bones[boneIDs[1]] * weights[1]);
	boneTransform += (bones[boneIDs[2]] * weights[2]);
	boneTransform += (bones[boneIDs[3]] * weights[3]);
	
	vec4 skinnedPosition = boneTransform * vec4(position, 1.0);
	gl_Position = projection * view * model * vec4(skinnedPosition.xyz, 1.0); 


	fragPos = vec3(model * vec4(skinnedPosition.xyz, 1.0));
	Normal = normalMat * normal;

	vec3 viewDir = normalize(viewPos - fragPos);
	detail = pow((length(normal * viewDir)), magnitude);
}