#version 330 core


struct DirLight
{
	vec3 direction;
	vec3 intensity;
};

#define NR_POINT_LIGHTS 1

in vec3 FragPosition;
in vec3 Normal;
in vec2 TexCoords;

out vec4 color;

uniform vec3 viewPos;
uniform DirLight dirLight;


// Function prototypes
vec3 calcDirLight(DirLight light, vec3 normal, vec3 fragPos, vec3 viewDir, vec3 diffColor, vec3 specColor);

void main()
{    
	vec3 result;
	vec3 viewDir = normalize(viewPos - FragPosition);
	vec3 norm = normalize(Normal);
	
	float height = FragPosition.y;
	float maxHeight = 100.0f;
	const float lowLevelEnd = 7.0f;
	const float midLevelStart = 22.0f;
	const float midLevelEnd = 24.0f;
	const float highLevelStart = 54.0f;
	
	
	const vec3 lowDiff = vec3(0.144f, 0.034f, 0.008f);
	const vec3 lowSpec = vec3(0.164f, 0.054f, 0.028f);
	const vec3 midDiff = vec3(0.021f, 0.118f, 0.022f);
	const vec3 midSpec = vec3(0.041f, 0.138f, 0.042f);
	//const vec3 highDiff = vec3(0.5f, 0.6f, 0.45f);
	//const vec3 highSpec = vec3(0.52f, 0.62f, 0.47f);
	const vec3 highDiff = vec3(0.3f, 0.4f, 0.25f);
	const vec3 highSpec = vec3(0.32f, 0.42f, 0.27f);
	
	if(height < lowLevelEnd)
	{
		result = calcDirLight(dirLight, norm, FragPosition, viewDir, lowDiff, lowSpec);
	}
	else if ( lowLevelEnd <= height &&  height < highLevelStart) //transition
	{
		float a = height - lowLevelEnd;
		a /= (highLevelStart - lowLevelEnd);
		result = calcDirLight(dirLight, norm, FragPosition, viewDir, mix(lowDiff, highDiff, a), mix(lowSpec, highSpec, a));
	}
	/*else if (lowLevelEnd <= height &&  height < midLevelStart) //transition
	{
		float a = height - lowLevelEnd;
		a /= (midLevelStart - lowLevelEnd);
		result = calcDirLight(dirLight, norm, FragPosition, viewDir, mix(lowDiff, midDiff, a), mix(lowSpec, midSpec, a));
	}
	else if (midLevelStart <= height && height < midLevelEnd)
	{
		result = calcDirLight(dirLight, norm, FragPosition, viewDir, midDiff, midSpec);
	}
	else if (midLevelEnd <= height && height < highLevelStart) //transition
	{
		float a = height - midLevelEnd;
		a /= (highLevelStart - midLevelEnd);
		result = calcDirLight(dirLight, norm, FragPosition, viewDir, mix(midDiff, highDiff, a), mix(midSpec, highSpec, a));
	}*/
	else if (highLevelStart<= height)
	{
		result = calcDirLight(dirLight, norm, FragPosition, viewDir, highDiff, highSpec);
	}

		
	color = vec4(result, 1.0f);
}


// Calculates the color when using a dir light.
vec3 calcDirLight(DirLight light, vec3 normal, vec3 fragPos, vec3 viewDir, vec3 diffColor, vec3 specColor)
{
	vec3 lightDir = normalize(-light.direction);
	// Diffuse shading
	float diff = max(dot(normal, lightDir), 0.0);
	// Specular shading
	vec3 reflectDir = reflect(-lightDir, normal);
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), 40);
	// Attenuation
	//float distance = length(light.position - fragPos);
	//float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));    
	// Combine results
	vec3 ambient = light.intensity * diffColor;
	vec3 diffuse = light.intensity * diff * diffColor;
	vec3 specular = light.intensity * spec * specColor;
	//ambient *= attenuation;
	//diffuse *= attenuation;
	//specular *= attenuation;
	return (ambient + diffuse + specular);
}